"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CreateTaskCommand = void 0;
const core_1 = require("@loopback/core");
const bpmn_service_1 = require("@sourceloop/bpmn-service");
const core_2 = require("@sourceloop/core");
const loopback4_authentication_1 = require("loopback4-authentication");
const keys_1 = require("../keys");
const types_1 = require("../types");
class CreateTaskCommand {
    constructor(context) {
        this.context = context;
        this.topic = 'create-tasks';
        this.logger = context.getSync(core_2.LOGGER.LOGGER_INJECT);
    }
    async execute() {
        const job = this.parameters.taskService;
        const variables = this.parameters.task.variables.getAll();
        const tempContext = new core_1.Context(this.context);
        const systemUser = await tempContext.get(keys_1.TaskServiceBindings.SYSTEM_USER);
        tempContext.bind(loopback4_authentication_1.AuthenticationBindings.CURRENT_USER).to(systemUser);
        try {
            const tasksWithWorkflow = JSON.parse(variables.tasks.values);
            let createdTasks = [];
            if (tasksWithWorkflow) {
                createdTasks = await this._createTasks(tasksWithWorkflow, tempContext);
            }
            const outgoing = this.context.getSync(keys_1.TaskServiceBindings.OUTGOING_CONNECTOR);
            await outgoing.publish({
                key: types_1.EventType.TaskCreated,
                payload: { createdTasks },
                timestamp: Date.now(),
                source: types_1.Source.TaskService,
            });
            await job.complete(this.parameters.task);
        }
        catch (e) {
            await job.handleFailure(this.parameters.task, {
                errorMessage: e.message,
                errorDetails: e.stack,
            });
        }
        finally {
            tempContext.close();
        }
    }
    async _createTasks(tasks, context) {
        const taskRepo = context.getSync('repositories.TaskRepository');
        const taskWorkflowMappingRepo = context.getSync('repositories.TaskWorkFlowRepository');
        let createdTasks = [];
        if (tasks.length) {
            createdTasks = await taskRepo.createAll(tasks !== null && tasks !== void 0 ? tasks : []);
        }
        const workflowKeys = await taskWorkflowMappingRepo.find({
            where: {
                taskKey: {
                    inq: createdTasks.map(task => task.key),
                },
            },
        });
        const workflowKeyMap = new Map();
        createdTasks.forEach(task => {
            var _a;
            const workflowKey = (_a = workflowKeys.find(key => key.taskKey === task.key)) === null || _a === void 0 ? void 0 : _a.workflowKey;
            if (workflowKey) {
                const mappedTasks = workflowKeyMap.get(workflowKey);
                if (mappedTasks) {
                    mappedTasks.push(task);
                }
                else {
                    workflowKeyMap.set(workflowKey, [task]);
                }
            }
        });
        const promises = Array.from(workflowKeyMap.entries()).map(([workflowKey, tsks]) => this._startWorkflowsForTasks(workflowKey, tsks !== null && tsks !== void 0 ? tsks : [], context));
        await Promise.all(promises);
        return createdTasks;
    }
    async _startWorkflowsForTasks(workflowKey, tasks, context) {
        const workflowCtrl = await context.get('controllers.WorkflowController');
        const workflowRepo = await context.get('repositories.WorkflowRepository');
        const taskRepo = await context.get('repositories.TaskRepository');
        const userTaskService = await context.get(keys_1.TaskServiceBindings.SUB_TASK_SERVICE);
        const workflow = await workflowRepo.findOne({
            where: {
                externalIdentifier: workflowKey,
            },
        });
        if (!workflow) {
            this.logger.debug(`No workflow found for key ${workflowKey}`);
            return;
        }
        const promises = tasks.map(async (dbTask) => {
            const result = await workflowCtrl.startWorkflow(workflow.id, new bpmn_service_1.ExecuteWorkflowDto({
                input: {
                    taskId: dbTask.id,
                },
            }));
            await userTaskService.updateList(dbTask.id, result['id']);
            await taskRepo.updateById(dbTask.id, {
                externalId: result['id'],
            });
        });
        await Promise.all(promises);
    }
}
exports.CreateTaskCommand = CreateTaskCommand;
//# sourceMappingURL=create-task.js.map